{% extends "wagtailadmin/admin_base.html" %}
{% load wagtailadmin_tags %}
{% load i18n %}
{% block furniture %}
    <div class="nav-wrapper">
        <div class="inner">
            <a href="{% url 'wagtailadmin_home' %}" class="logo"><img src="{{ STATIC_URL }}wagtailadmin/images/wagtail-logo.svg" alt="Wagtail" width="80" /><span>{% trans "Dashboard" %}</span></a>
            {% main_nav %}
        </div>

        <nav id="explorer" class="explorer"></nav>
    </div>

    <div class="content-wrapper">
        <div class="content">
            {# Always show messages div so it can be appended to by JS #}
            <div class="messages">
                {% if messages %}
                    <ul>
                        {% for message in messages %}
                            <li{% if message.tags %} class="{{ message.tags }}"{% endif %}>{{ message|safe }}</li>
                        {% endfor %}
                    </ul>
                {% endif %}
            </div>

            <div id="nav-toggle" class="icon text-replace">{% trans "Menu" %}</div>

            {% block content %}{% endblock %}
        </div>
    </div>
{% endblock %}