{% load i18n %}
<div class="wagtail-moderator-controls">
    <div class="message">
        <p>{% blocktrans with title=revision.page.title submitted_by=revision.user.get_full_name|default:revision.user.username submitted_on=revision.created_at|date:"d M Y H:i" %}
            Previewing '{{ title }}', submitted by {{ submitted_by }} on {{ submitted_on }}.
        {% endblocktrans %}</p>
    </div>
    <div class="actions">
        <a href="{% url 'wagtailadmin_pages_edit' revision.page.id %}" class="button">{% trans 'Edit' %}</a>
        <form action="{% url 'wagtailadmin_pages_approve_moderation' revision.id %}" method="POST">
            {% csrf_token %}
            <input type="submit" class="button yes" value="{% trans 'Approve' %}" />
        </form>

        <form action="{% url 'wagtailadmin_pages_reject_moderation' revision.id %}" method="POST">
            {% csrf_token %}
            <input type="submit" class="button no" value="{% trans 'Reject' %}" />
        </form>
    </div>
</div>
