{% extends "wagtailadmin/skeleton.html" %}
{% load compress %}

{% block css %}
    {% compress css %}
        <link rel="stylesheet" href="{{ STATIC_URL }}wagtailadmin/scss/vendor/jquery-ui/jquery-ui-1.10.3.verdant.css" />
        <link rel="stylesheet" href="{{ STATIC_URL }}wagtailadmin/scss/core.scss" type="text/x-scss" />
    {% endcompress %}

    {% block extra_css %}{% endblock %}
{% endblock %}

{% block js %}
    
    {% compress js %}
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/jquery-1.10.3.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/jquery-ui-1.10.3.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/jquery.datetimepicker.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/jquery.autosize.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/bootstrap-transition.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/bootstrap-modal.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/bootstrap-tab.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/vendor/jquery.dlmenu.js"></script>
        <script src="{{ STATIC_URL }}wagtailadmin/js/core.js"></script>
        <script>
            window.explorer_menu_url = "{% url 'wagtailadmin_explorer_nav' %}";
        </script>
    {% endcompress %}
    
    {% block extra_js %}{% endblock %}
{% endblock %}